import multiprocessing
from os import path
import os
import platform
import subprocess
from zipfile import ZipFile


HERE = path.dirname(path.realpath(__file__))
SHADERC_ZIP = path.join(HERE, 'shaderc-cb4f0f6.zip')
SHADERC_SRC = path.join(HERE, 'shaderc')
SHADERC_BIN = path.join(HERE, 'shaderc_build')
STATIC_LIB_FOLDER = path.join(SHADERC_BIN, 'libshaderc')


# ----------
# BUILD STATIC LIB
# ----------
def build():
    # Extract shaderc
    if not path.exists(SHADERC_SRC):
        z = ZipFile(SHADERC_ZIP)
        z.extractall(HERE)
        z.close()

    if not path.exists(SHADERC_BIN):
        os.makedirs(SHADERC_BIN)

    # Prepare
    shell = platform.system() == 'Windows'
    options = [
        '-DCMAKE_BUILD_TYPE=Release',
        '-DCMAKE_POSITION_INDEPENDENT_CODE=ON',
        '-DSPIRV_SKIP_EXECUTABLES=ON',
        '-DSHADERC_SKIP_TESTS=ON'
    ]

    if platform.system() == 'Windows':
        options += [
            '-DCMAKE_C_FLAGS=/nologo /EHsc /MD',
            '-DCMAKE_CXX_FLAGS=/nologo /EHsc /MD',
            '-DCMAKE_C_FLAGS_RELEASE=/nologo /EHsc /MD',
            '-DCMAKE_CXX_FLAGS_RELEASE=/nologo /EHsc /MD',
            '-DSHADERC_ENABLE_SHARED_CRT=ON'
        ]

    call = ['cmake', '-B'+SHADERC_BIN, '-H'+SHADERC_SRC]
    call += options
    subprocess.check_call(call, stderr=subprocess.STDOUT, shell=shell)

    # Build
    cpu = ''
    if platform.system() == 'Linux':
        cpu = '-j' + str(multiprocessing.cpu_count() * 2)

    subprocess.check_call(['cmake', '--build', SHADERC_BIN, '--config',
                          'Release', '--', cpu], stderr=subprocess.STDOUT,
                          shell=shell)


if __name__ == '__main__':
    build()



from distutils.command.build import build
from setuptools import setup

from _cffi_build import shaderc_build


class ShadercBuild(build):
    def run(self):
        shaderc_build.build()
        super(ShadercBuild, self).run()


setup(
    name="pyshaderc",
    version='1.1.1',  # can't use pyshader.__version__
    author="realitix",
    author_email="realitix@gmail.com",
    description="Python CFFI binding for shaderc",
    long_description="Python CFFI binding for shaderc",
    packages=['_cffi_build', 'pyshaderc'],
    install_requires=["cffi"],
    setup_requires=["cffi"],
    include_package_data=True,
    url="http://github.com/realitix/pyshaderc",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.5",
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    license="Apache 2.0",
    ext_package="pyshaderc",
    cmdclass={'build': ShadercBuild},
    cffi_modules=["_cffi_build/pyshaderc_build.py:ffi"]
)
